// Inverse Kinematics Application

// Link with: opengl32.lib, glu32.lib, glut32.lib.

#ifdef WIN32
#define NOMINMAX
#include <Windows.h>
#endif

#include <GL/glut.h>
#include <math.h>
#include <iostream>
#include <strstream>
#include <iomanip>
#include "linkedstructure.h"
#include "link.h"
#include "Eigen/Dense"

using namespace Eigen;
using namespace std;

// Current size of window.
int width  = 600;
int height = 400;

// Bounds of viewing frustum.
float nearPlane =  1.0;
float farPlane  = 1000.0;

// Viewing angle.
double fovy = 40.0;

LinkedStructure linkedstructure;


// This function is called to display the scene.
void display () {

    glDisable(GL_LIGHTING);
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glOrtho(0.0f, width, height, 0.0f, 0.0f, 1.0f);

    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
	glTranslatef(width/2, height/2, 0.0);

    linkedstructure.draw();
    
    glFlush();
    glutSwapBuffers();
}

// This function is called when there is nothing else to do.
void idle()
{
    glutPostRedisplay();
}

// This function gets called every 10ms
void timer(int i)
{
	linkedstructure.update();
	glutTimerFunc(10, timer, i);
	glutPostRedisplay();
}

// Respond to window resizing, preserving proportions.
// Parameters give new window size in pixels.
void reshapeMainWindow (int newWidth, int newHeight)
{
    width = newWidth;
    height = newHeight;
    glViewport(0, 0, width, height);
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    gluPerspective(fovy, GLfloat(width) / GLfloat(height), nearPlane, farPlane);
}

// Respond to graphic character keys.
// Parameters give key code and mouse coordinates.
void graphicKeys (unsigned char key, int x, int y)
{
    switch (key)
    {
	    case 27:
		    exit(0);
			break;
		case 'n':
			cout<<"next target"<<endl;
			linkedstructure.GenerateTarget(); 
			break;
    }
}



int main (int argc, char **argv)
{

	// Constructing the linked structure by adding links
    Link *li;
	Color c = {1.0f, 0.0f, 1.0f, 1.0f};
	li = new Link(c);
    li->setRotation(3.1415f/5.0);
    li->mTranslation = 0;
    linkedstructure.addLink(li);
	c.r = 0.0;
	li = new Link(c);
    li->setRotation(3.1415f/4.0);
    li->mTranslation = 50;
    linkedstructure.addLink(li);
	c.b = 0.0;
	c.g = 1.0;
	li = new Link(c);
    li->setRotation(3.1415f/4.0);
    li->mTranslation = 50;
    linkedstructure.addLink(li);
	c.r = 0.5;
	c.b = 1.0;
	c.g = 0.5;
	li = new Link(c);
    li->setRotation(0.0);
    li->mTranslation = 50;
    linkedstructure.addLink(li);
	/*
	c.r = 1.0;
	c.b = 1.0;
	c.g = 0.5;
	li = new Link(c);
    li->setRotation(0.0);
    li->mTranslation = 50;
    linkedstructure.addLink(li);
	*/





    // GLUT initialization.
    glutInit(&argc, argv);
    glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGBA | GLUT_DEPTH);
    glutInitWindowSize(width, height);
    glutCreateWindow("Inverse Kinematics Arm");

    // Register call backs.
    glutDisplayFunc(display);
    glutReshapeFunc(reshapeMainWindow);
    glutKeyboardFunc(graphicKeys);
    glutIdleFunc(idle);
    glutTimerFunc(60, timer, 0);

    // OpenGL initialization
    glEnable(GL_DEPTH_TEST);
    glEnable(GL_NORMALIZE);
    glShadeModel(GL_SMOOTH);
    glEnable(GL_LIGHTING);
    glEnable(GL_LIGHT0);
    glEnable(GL_COLOR_MATERIAL);

    GLfloat global_ambient[] = { 1.0f, 1.0f, 1.0f, 1.0f };
    glLightModelfv(GL_AMBIENT_AND_DIFFUSE, global_ambient);

    // Enter GLUT loop.
    glutMainLoop();

    return 0;
}

